from checkov.terraform.checks.provider.aws import *
from checkov.terraform.checks.provider.linode import *
from checkov.terraform.checks.provider.bridgecrew import *
from checkov.terraform.checks.provider.oci import *
from checkov.terraform.checks.provider.openstack import *
